// phong_spot.frag
//
// Phong fragment shader for a single spotlight light source. All lighting
// calculations are performed in eye space. In eye space the origin is the
// camera's position. Performing all lighting in eye space means that we don't
// require the application to pass in the camera's position each time it uses
// this shader.

#version 130

// Precision qualifiers are added for code portability with OpenGL ES, not for
// functionality. According to the GLSL 1.30 and later specs: the same object
// declared in different shaders that are linked together must have the same
// precision qualification. This applies to inputs, outputs, uniforms, and
// globals.

precision highp float;

// Custom user defined structures used to model the spotlight light source
// and the characteristics of the material that the light will interact with.
// These are setup in the host application. This must match the declaration in
// the vertex shader.

uniform struct Spotlight
{
	float range;
	float cosInnerCone;
	float cosOuterCone;
	vec3 position;
	vec3 direction;
	vec4 ambient;
	vec4 diffuse;
	vec4 specular;
} Spotlight1;

uniform struct Material
{
	vec4 ambient;
	vec4 diffuse;
	vec4 specular;
	float shininess;
} Material1;

uniform sampler2D ColorMapSampler;

// Fragment shader input variable declarations must exactly match the vertex
// shader's output variable declarations. The output of the vertex shader and
// the input of the fragment shader form an interface. For this interface,
// vertex shader output variables and fragment shader input variables of the
// same name must match in type and qualification (other than out matching to
// in).

in vec3 VertexPos;
in vec3 LightDir;
in vec3 SpotlightDir;
in vec3 Normal;
in vec2 TexCoord;

// GLSL 1.3 deprecates gl_FragColor. Its replacement is user defined output
// variables. Fragment shader output variables can be explicitly bound to a
// render target. When a program is linked any output variables without an
// explicit binding specified through glBindFragDataLocation() will
// automatically be bound to fragment colors by the GL.

out vec4 FragColor;

void main()
{
	float atten = max(0.0, 1.0 - dot(LightDir, LightDir));

	vec3 L = normalize(LightDir);
	vec3 SL = normalize(SpotlightDir);
	vec3 N = normalize(Normal);
	vec3 V = normalize(-VertexPos);
	vec3 R = normalize(-reflect(L, N));

	float spotDot = dot(-L, SL);
	float spotEffect = smoothstep(Spotlight1.cosOuterCone, Spotlight1.cosInnerCone, spotDot);
	
	atten *= spotEffect;
	
	float nDotL = max(0.0, dot(N, L));
	float rDotV = max(0.0, dot(R, V));
	
	vec4 ambient = Spotlight1.ambient * Material1.ambient * atten;
	vec4 diffuse = Spotlight1.diffuse * Material1.diffuse * nDotL * atten;
	vec4 specular = Spotlight1.specular * Material1.specular * pow(rDotV, Material1.shininess) * atten;
	vec4 texel = texture(ColorMapSampler, TexCoord);
	
    FragColor = (ambient + diffuse + specular) * texel;
}
