// phong_spot.vert
//
// Phong vertex shader for a single spotlight light source. All lighting
// calculations are performed in eye space. In eye space the origin is the
// camera's position. Performing all lighting in eye space means that we don't
// require the application to pass in the camera's position each time it uses
// this shader.

#version 130

// Precision qualifiers are added for code portability with OpenGL ES, not for
// functionality. According to the GLSL 1.30 and later specs: the same object
// declared in different shaders that are linked together must have the same
// precision qualification. This applies to inputs, outputs, uniforms, and
// globals.

precision highp float;

// Custom user defined structures used to model the spotlight light source
// and the characteristics of the material that the light will interact with.
// These are setup in the host application.

uniform struct Spotlight
{
	float range;
	float cosInnerCone;
	float cosOuterCone;
	vec3 position;
	vec3 direction;
	vec4 ambient;
	vec4 diffuse;
	vec4 specular;
} Spotlight1;

uniform struct Material
{
	vec4 ambient;
	vec4 diffuse;
	vec4 specular;
	float shininess;
} Material1;

// GLSL 1.30 (OpenGL 3.0) deprecates most of the built-in uniform state
// variables. In GLSL 1.40 (OpenGL 3.1) they have been removed. Typically we
// need to supply projection, view, and model matrices in order to transform
// vertices from model space to clip space.

uniform mat4 ProjectionMatrix;
uniform mat4 ViewMatrix;
uniform mat4 ModelMatrix;
uniform mat3 NormalMatrix;

// GLSL 1.30 (OpenGL 3.0) deprecates vertex shader attribute variables. They
// have been replaced with user-defined generic shader input variables. The
// application code uses the new glVertexAttribPointer() function to map vertex
// data stored in a vertex buffer object to these generic vertex shader input
// variables. This replaces the existing method of specifying vertex array data
// by calling glVertexPointer(), glNormalPointer(), etc.

in vec3 VertexPosition;
in vec3 VertexNormal;
in vec2 VertexTexCoord;

// GLSL 1.30 (OpenGL 3.0) also deprecates varying shader variables. This has
// been replaced with user-defined generic shader output variables. The output
// of the vertex shader and the input of the fragment shader form an interface.
// For this interface, vertex shader output variables and fragment shader input
// variables of the same name must match in type and qualification (other than
// out matching to in).

out vec3 VertexPos;
out vec3 LightDir;
out vec3 SpotlightDir;
out vec3 Normal;
out vec2 TexCoord;

void main()
{
	// All vertex shaders should write the transformed homogeneous clip space
	// vertex position into the gl_Position variables.

	mat4 modelViewMatrix = ViewMatrix * ModelMatrix;
    mat4 mvpMatrix = ProjectionMatrix * modelViewMatrix;
    vec4 pos = vec4(VertexPosition.x, VertexPosition.y, VertexPosition.z, 1.0);
    
    gl_Position = mvpMatrix * pos;
    
    // Transform the vertex position into eye space. We use this later on to
    // calculate the view (eye) vector.

    pos = modelViewMatrix * pos;
	VertexPos = pos.xyz / pos.w;

	// Transform the spotlight position into eye space.
	// Spotlights are specified in world space.
	
	pos = ViewMatrix * vec4(Spotlight1.position, 1.0);
	
	vec3 spotlightPosEye = pos.xyz / pos.w;
        
    // Calculate the light direction. This is the direction from the vertex
    // position to the spotlight position. The division by the spotlight range
    // is used for the attenuation calculation in the fragment shader.
	
	LightDir = (spotlightPosEye - VertexPos) / Spotlight1.range;
    
    // Transform the spotlight direction into eye space.
    // This is the direction that the spotlight is pointing.
    
    SpotlightDir = vec3(ViewMatrix * vec4(Spotlight1.direction, 0.0));
    
    Normal = NormalMatrix * VertexNormal;
    TexCoord = VertexTexCoord;
}
